package cn.com.qmth.examcloud.core.basic.service.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import org.apache.commons.fileupload.disk.DiskFileItem;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import cn.com.qmth.examcloud.api.commons.exchange.FormFilePart;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.ResourceRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.ResourceEntity;
import cn.com.qmth.examcloud.core.basic.service.ResourceService;
import cn.com.qmth.examcloud.core.basic.service.bean.ResourceInfo;
import cn.com.qmth.examcloud.exchange.inner.api.UpyunCloudService;
import cn.com.qmth.examcloud.exchange.inner.api.request.PutFileReq;

@Service
public class ResourceServiceImpl implements ResourceService {

	private static final String FILE_ROOT_PATH = "/resource";

	private static final String FILE_PATH_SEPARATE = "/";

	@Autowired
	ResourceRepo resourceRepo;

	@Autowired
	UpyunCloudService upyunCloudService;

	@Override
	public void addDir(ResourceInfo info) {

		ResourceEntity e = new ResourceEntity();
		e.setName(info.getName());
		e.setRootOrgId(info.getRootOrgId());
		e.setIsFile(false);
		if (info.getParentId() == null) {
			e.setParentId(null);
			e.setFilePath(FILE_ROOT_PATH + FILE_PATH_SEPARATE + info.getRootOrgId()
					+ FILE_PATH_SEPARATE + info.getName());
		} else {
			e.setParentId(info.getParentId());
			Optional<ResourceEntity> op = resourceRepo.findById(info.getParentId());
			if (op.isPresent()) {
				ResourceEntity pe = op.get();
				e.setFilePath(pe.getFilePath() + FILE_PATH_SEPARATE + info.getName());
			} else {
				throw new StatusException("100000", "上级目录不存在");
			}
		}
		resourceRepo.saveAndFlush(e);
	}

	@Override
	public void addFile(Long rootOrgId, Long parentId, MultipartFile dataFile) {

		ResourceEntity e = new ResourceEntity();
		String fileName = dataFile.getOriginalFilename();
		e.setName(fileName);
		e.setRootOrgId(rootOrgId);
		e.setIsFile(true);
		String suffix = fileName.substring(fileName.lastIndexOf(".") + 1, fileName.length());
		e.setSuffix(suffix);
		if (parentId == null) {
			e.setParentId(null);
			e.setFilePath(FILE_ROOT_PATH + FILE_PATH_SEPARATE + rootOrgId + FILE_PATH_SEPARATE
					+ fileName);
		} else {
			e.setParentId(parentId);
			Optional<ResourceEntity> op = resourceRepo.findById(parentId);
			if (op.isPresent()) {
				ResourceEntity pe = op.get();
				e.setFilePath(pe.getFilePath() + FILE_PATH_SEPARATE + fileName);
			} else {
				throw new StatusException("100020", "上级目录不存在");
			}
		}
		ResourceEntity olde = resourceRepo.findByRootOrgIdAndFilePath(rootOrgId, e.getFilePath());
		if (olde != null) {
			olde.setUpdateTime(new Date());
			resourceRepo.save(olde);
		} else {
			resourceRepo.save(e);
		}
		PutFileReq req = new PutFileReq();
		List<FormFilePart> formFilePartList = new ArrayList<FormFilePart>();
		CommonsMultipartFile cf = (CommonsMultipartFile) dataFile;
		DiskFileItem fi = (DiskFileItem) cf.getFileItem();

		File f = fi.getStoreLocation();
		FormFilePart part = new FormFilePart("file", fileName, f);
		formFilePartList.add(part);

		req.setFormFilePartList(formFilePartList);
		req.setSiteId("resource");
		req.setRootOrgId(rootOrgId);
		String relativePath = e.getFilePath().replaceFirst(FILE_ROOT_PATH, "")
				.replaceFirst(FILE_PATH_SEPARATE, "");
		req.setRelativePath(relativePath);
		upyunCloudService.putFile(req);
	}
}
